@@ -208,10 +208,9 @@ read.graph.edgelist <- function(file, n=0,
   if (length(list(...))>0) {
     stop("Unknown arguments to read_graph (edgelist format)")
   }
-  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_read_graph_edgelist", file,
-        as.numeric(n), as.logical(directed),
-        PACKAGE="igraph")
+  on.exit( .Call(C_R_igraph_finalizer) )
+  .Call(C_R_igraph_read_graph_edgelist, file,
+        as.numeric(n), as.logical(directed))
 }
 
 write.graph.edgelist <- function(graph, file, ...) {
@@ -219,9 +218,8 @@ write.graph.edgelist <- function(graph, file, ...) {
   if (length(list(...))>0) {
     stop("Unknown arguments to write_graph (edgelist format)")
   }
-  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_write_graph_edgelist", graph, file,
-        PACKAGE="igraph")
+  on.exit( .Call(C_R_igraph_finalizer) )
+  .Call(C_R_igraph_write_graph_edgelist, graph, file)
 }
 
 ################################################################
@@ -236,10 +234,9 @@ read.graph.ncol <- function(file, predef=character(0), names=TRUE,
     stop("Unknown arguments to read_graph (NCOL format)")
   }
   weights <- switch(igraph.match.arg(weights), "no"=0, "yes"=1, "auto"=2)
-  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_read_graph_ncol", file, as.character(predef),
-        as.logical(names), as.numeric(weights), as.logical(directed),
-        PACKAGE="igraph")
+  on.exit( .Call(C_R_igraph_finalizer) )
+  .Call(C_R_igraph_read_graph_ncol, file, as.character(predef),
+        as.logical(names), as.numeric(weights), as.logical(directed))
 }
 
 write.graph.ncol <- function(graph, file, 
@@ -252,10 +249,9 @@ write.graph.ncol <- function(graph, file,
   if (length(names)==0 || ! names %in% vertex_attr_names(graph)) { names <- NULL }
   if (length(weights)==0 || ! weights %in% edge_attr_names(graph)) { weights <- NULL }
   
-  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_write_graph_ncol", graph, file,
-        names, weights,
-        PACKAGE="igraph")
+  on.exit( .Call(C_R_igraph_finalizer) )
+  .Call(C_R_igraph_write_graph_ncol, graph, file,
+        names, weights)
 }  
 
 read.graph.lgl <- function(file, names=TRUE,
@@ -267,10 +263,9 @@ read.graph.lgl <- function(file, names=TRUE,
     stop("Unknown arguments to read_graph (LGL format)")
   }
   weights <- switch(igraph.match.arg(weights), "no"=0, "yes"=1, "auto"=2)
-  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_read_graph_lgl", file,
-        as.logical(names), as.numeric(weights), as.logical(directed),
-        PACKAGE="igraph")
+  on.exit( .Call(C_R_igraph_finalizer) )
+  .Call(C_R_igraph_read_graph_lgl, file,
+        as.logical(names), as.numeric(weights), as.logical(directed))
 }
 
 write.graph.lgl <- function(graph, file, 
@@ -284,10 +279,9 @@ write.graph.lgl <- function(graph, file,
   if (length(names)==0 || ! names %in% vertex_attr_names(graph)) { names <- NULL }
   if (length(weights)==0 || ! weights %in% edge_attr_names(graph)) { weights <- NULL }
   
-  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_write_graph_lgl", graph, file,
-        names, weights, as.logical(isolates),
-        PACKAGE="igraph")
+  on.exit( .Call(C_R_igraph_finalizer) )
+  .Call(C_R_igraph_write_graph_lgl, graph, file,
+        names, weights, as.logical(isolates))
 }  
 
 read.graph.pajek <- function(file, ...) {
@@ -295,9 +289,8 @@ read.graph.pajek <- function(file, ...) {
   if (length(list(...))>0) {
     stop("Unknown arguments to read_graph (Pajek format)")
   }
-  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  res <- .Call("R_igraph_read_graph_pajek", file,
-               PACKAGE="igraph")
+  on.exit( .Call(C_R_igraph_finalizer) )
+  res <- .Call(C_R_igraph_read_graph_pajek, file)
   if ("type" %in% vertex_attr_names(res)) {
     type <- as.logical(V(res)$type)
     res <- delete_vertex_attr(res, "type")
@@ -311,9 +304,8 @@ write.graph.pajek <- function(graph, file, ...) {
   if (length(list(...))>0) {
     stop("Unknown arguments to write_graph (Pajek format)")
   }
-  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_write_graph_pajek", graph, file,
-        PACKAGE="igraph")
+  on.exit( .Call(C_R_igraph_finalizer) )
+  .Call(C_R_igraph_write_graph_pajek, graph, file)
 }
 
 read.graph.dimacs <- function(file, directed=TRUE, ...) {
@@ -321,8 +313,7 @@ read.graph.dimacs <- function(file, directed=TRUE, ...) {
   if (length(list(...))>0) {
     stop("Unknown arguments to read_graph (DIMACS format)")
   }
-  res <- .Call("R_igraph_read_graph_dimacs", file, as.logical(directed),
-               PACKAGE="igraph")
+  res <- .Call(C_R_igraph_read_graph_dimacs, file, as.logical(directed))
   if (res[[1]][1] == "max") {
     graph <- res[[2]]
     graph <- set_graph_attr(graph, "problem", res[[1]])
@@ -354,10 +345,9 @@ write.graph.dimacs <- function(graph, file,
     capacity <- E(graph)$capacity
   }
   
-  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_write_graph_dimacs", graph, file, as.numeric(source),
-        as.numeric(target), as.numeric(capacity),
-        PACKAGE="igraph")
+  on.exit( .Call(C_R_igraph_finalizer) )
+  .Call(C_R_igraph_write_graph_dimacs, graph, file, as.numeric(source),
+        as.numeric(target), as.numeric(capacity))
 }
 
 ################################################################
@@ -369,9 +359,8 @@ read.graph.graphml <- function(file, index=0, ...) {
   if (length(list(...))>0) {
     stop("Unknown arguments to read_graph (GraphML format)")
   }
-  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_read_graph_graphml", file, as.numeric(index),
-        PACKAGE="igraph")
+  on.exit( .Call(C_R_igraph_finalizer) )
+  .Call(C_R_igraph_read_graph_graphml, file, as.numeric(index))
 }
 
 write.graph.graphml <- function(graph, file, prefixAttr=TRUE, ...) {
@@ -379,9 +368,8 @@ write.graph.graphml <- function(graph, file, prefixAttr=TRUE, ...) {
   if (length(list(...))>0) {
     stop("Unknown arguments to write_graph (GraphML format)")
   }
-  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_write_graph_graphml", graph, file, as.logical(prefixAttr),
-        PACKAGE="igraph")
+  on.exit( .Call(C_R_igraph_finalizer) )
+  .Call(C_R_igraph_write_graph_graphml, graph, file, as.logical(prefixAttr))
 }
 
 ################################################################
@@ -392,9 +380,8 @@ read.graph.gml <- function(file, ...) {
   if (length(list(...))>0) {
     stop("Unknown arguments to read_graph (GML format)")
   }
-  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_read_graph_gml", file,
-        PACKAGE="igraph")
+  on.exit( .Call(C_R_igraph_finalizer) )
+  .Call(C_R_igraph_read_graph_gml, file)
 }
 
 write.graph.gml <- function(graph, file, id=NULL, creator=NULL, ...) {
@@ -407,9 +394,8 @@ write.graph.gml <- function(graph, file, id=NULL, creator=NULL, ...) {
   if (!is.null(creator)) {
     creator <- as.character(creator)
   }
-  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_write_graph_gml", graph, file, id, creator,
-        PACKAGE="igraph")
+  on.exit( .Call(C_R_igraph_finalizer) )
+  .Call(C_R_igraph_write_graph_gml, graph, file, id, creator)
 }
 
 ################################################################
@@ -420,9 +406,8 @@ read.graph.dl <- function(file, directed=TRUE, ...) {
   if (length(list(...))>0) {
     stop("Unknown arguments to read_graph (DL format)")
   }
-  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_read_graph_dl", file, as.logical(directed),
-        PACKAGE="igraph")
+  on.exit( .Call(C_R_igraph_finalizer) )
+  .Call(C_R_igraph_read_graph_dl, file, as.logical(directed))
 }  
 
 ################################################################
@@ -433,9 +418,8 @@ write.graph.dot <- function(graph, file, ...) {
   if (length(list(...))>0) {
     stop("Unknown arguments to write_graph (DOT format)")
   }
-  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_write_graph_dot", graph, file,
-        PACKAGE="igraph")
+  on.exit( .Call(C_R_igraph_finalizer) )
+  .Call(C_R_igraph_write_graph_dot, graph, file)
 }
 
 ################################################################
@@ -557,18 +541,15 @@ graph_from_graphdb <- function(url=NULL,
   f <- tempfile()
   write.graph.fromraw(buffer, f)
 
-  .Call("R_igraph_read_graph_graphdb", f, as.logical(directed),
-        PACKAGE="igraph")
-  
+  .Call(C_R_igraph_read_graph_graphdb, f, as.logical(directed))
 }
 
 read.graph.graphdb <- function(file, directed=TRUE, ...) {
   if (length(list(...))>0) {
     stop("Unknown arguments to read_graph (GraphDB format)")
   }
-  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_read_graph_graphdb", file, as.logical(directed),
-        PACKAGE="igraph")
+  on.exit( .Call(C_R_igraph_finalizer) )
+  .Call(C_R_igraph_read_graph_graphdb, file, as.logical(directed))
 }
 
 write.graph.leda <- function(graph, file, vertex.attr=NULL, edge.attr=NULL,
@@ -578,7 +559,6 @@ write.graph.leda <- function(graph, file, vertex.attr=NULL, edge.attr=NULL,
   }
   if (!is.null(vertex.attr)) { vertex.attr <- as.character(vertex.attr) }
   if (!is.null(edge.attr))   { edge.attr   <- as.character(edge.attr)   }
-  on.exit(.Call("R_igraph_finalizer", PACKAGE="igraph") )
-  .Call("R_igraph_write_graph_leda", graph, file, vertex.attr, edge.attr,
-        PACKAGE="igraph")
+  on.exit(.Call(C_R_igraph_finalizer) )
+  .Call(C_R_igraph_write_graph_leda, graph, file, vertex.attr, edge.attr)
 }
