@@ -41,7 +41,8 @@ int igraph_i_eccentricity(const igraph_t *graph,
   igraph_vit_t vit;
   igraph_vector_int_t counted;
   int i, mark=1;
-  igraph_vector_t vneis, *neis=&vneis;
+  igraph_vector_t vneis;
+  igraph_vector_int_t *neis;
 
   IGRAPH_CHECK(igraph_dqueue_long_init(&q, 100));
   IGRAPH_FINALLY(igraph_dqueue_long_destroy, &q);
@@ -53,7 +54,7 @@ int igraph_i_eccentricity(const igraph_t *graph,
   IGRAPH_FINALLY(igraph_vector_int_destroy, &counted);
 
   if (!adjlist) {
-    IGRAPH_VECTOR_INIT_FINALLY(neis, 0);
+    IGRAPH_VECTOR_INIT_FINALLY(&vneis, 0);
   }
 
   IGRAPH_CHECK(igraph_vector_resize(res, IGRAPH_VIT_SIZE(vit)));
@@ -82,24 +83,34 @@ int igraph_i_eccentricity(const igraph_t *graph,
 
       if (adjlist) {
 	neis=igraph_adjlist_get(adjlist, act);
+	n=(int) igraph_vector_int_size(neis);
+	for (j=0; j<n; j++) {
+	  int nei=(int) VECTOR(*neis)[j];
+	  if (VECTOR(counted)[nei] != mark) {
+	    VECTOR(counted)[nei]=mark;
+	    IGRAPH_CHECK(igraph_dqueue_long_push(&q, nei));
+	    IGRAPH_CHECK(igraph_dqueue_long_push(&q, dist+1));
+	  }
+	}  
       } else {
-	IGRAPH_CHECK(igraph_neighbors(graph, neis, act, mode));
+	IGRAPH_CHECK(igraph_neighbors(graph, &vneis,
+				      (igraph_integer_t) act, mode));
+	n=(int) igraph_vector_size(&vneis);
+	for (j=0; j<n; j++) {
+	  int nei=(int) VECTOR(vneis)[j];
+	  if (VECTOR(counted)[nei] != mark) {
+	    VECTOR(counted)[nei]=mark;
+	    IGRAPH_CHECK(igraph_dqueue_long_push(&q, nei));
+	    IGRAPH_CHECK(igraph_dqueue_long_push(&q, dist+1));
+	  }
+	}  
       }
-      n=igraph_vector_size(neis);
-      for (j=0; j<n; j++) {
-	int nei=VECTOR(*neis)[j];
-	if (VECTOR(counted)[nei] != mark) {
-	  VECTOR(counted)[nei]=mark;
-	  IGRAPH_CHECK(igraph_dqueue_long_push(&q, nei));
-	  IGRAPH_CHECK(igraph_dqueue_long_push(&q, dist+1));
-	}
-      }      
     } /* while !igraph_dqueue_long_empty(dqueue) */
 
   } /* for IGRAPH_VIT_NEXT(vit) */
 
   if (!adjlist) {
-    igraph_vector_destroy(neis);
+    igraph_vector_destroy(&vneis);
     IGRAPH_FINALLY_CLEAN(1);
   }
   igraph_vector_int_destroy(&counted);
