@@ -52,7 +52,7 @@
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 82 "igraph/src/foreign-ncol-parser.y"
+#line 87 "src/foreign-ncol-parser.y"
 {
   long int edgenum;
   double weightnum;
