@@ -1507,15 +1507,24 @@ int igraph_subisomorphic_lad(const igraph_t *pattern, const igraph_t *target,
 		 IGRAPH_EINVAL);
   }
 
+  if (igraph_is_directed(pattern) != igraph_is_directed(target)) {
+    IGRAPH_ERROR("Cannot search for a directed pattern in an undirected target "
+		 "or vice versa", IGRAPH_EINVAL);
+  }
   if (time_limit<=0) { time_limit = INT_MAX; }
     
-  IGRAPH_CHECK(igraph_i_lad_createGraph(pattern, &Gp));
-  IGRAPH_CHECK(igraph_i_lad_createGraph(target, &Gt));
-  
-  if (iso)  { *iso = 0; }
+  if (iso)  { *iso = (igraph_vcount(pattern) == 0); }
   if (map)  { igraph_vector_clear(map); } 
   if (maps) { igraph_vector_ptr_clear(maps); }
 
+  if (igraph_vcount(pattern) == 0) {
+    /* Special case for empty graphs */
+    return IGRAPH_SUCCESS;
+  }
+  
+  IGRAPH_CHECK(igraph_i_lad_createGraph(pattern, &Gp));
+  IGRAPH_CHECK(igraph_i_lad_createGraph(target, &Gt));
+  
   if (Gp.nbVertices > Gt.nbVertices) { goto exit3; }
   
   IGRAPH_CHECK(igraph_i_lad_initDomains(initialDomains, domains, &D, &Gp, 
